From 5f5b8510d86db59cb82ecf07ae9e7cb86937b8f2 Mon Sep 17 00:00:00 2001
From: Stefan Kangas <stefankangas@gmail.com>
Date: Wed, 8 May 2019 22:17:14 +0200
Subject: [PATCH] Use lexical-binding in tabify.el and add tests

* lisp/tabify.el: Use lexical-binding.

* test/lisp/tabify-tests.el: New file.
---
 lisp/tabify.el            |  2 +-
 test/lisp/tabify-tests.el | 79 +++++++++++++++++++++++++++++++++++++++++++++++
 2 files changed, 80 insertions(+), 1 deletion(-)
 create mode 100644 test/lisp/tabify-tests.el

diff --git a/lisp/tabify.el b/lisp/tabify.el
index 37cd8fb2fd..e2df8fc405 100644
--- a/lisp/tabify.el
+++ b/lisp/tabify.el
@@ -1,4 +1,4 @@
-;;; tabify.el --- tab conversion commands for Emacs
+;;; tabify.el --- tab conversion commands for Emacs  -*- lexical-binding: t -*-
 
 ;; Copyright (C) 1985, 1994, 2001-2019 Free Software Foundation, Inc.
 
diff --git a/test/lisp/tabify-tests.el b/test/lisp/tabify-tests.el
new file mode 100644
index 0000000000..6670702eac
--- /dev/null
+++ b/test/lisp/tabify-tests.el
@@ -0,0 +1,79 @@
+;;; tabify-tests.el --- tests for tabify.el  -*- lexical-binding: t -*-
+
+;; Copyright (C) 2019 Free Software Foundation, Inc.
+
+;; Author: Stefan Kangas <stefankangas@gmail.com>
+
+;; This file is part of GNU Emacs.
+
+;; GNU Emacs is free software: you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
+
+;; GNU Emacs is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
+
+;;; Code:
+
+(require 'ert)
+(require 'tabify)
+
+(defun tabify-tests--test-changes (fun changes tab-width)
+  (with-temp-buffer
+    (let ((tab-width tab-width))
+      (insert (apply 'concat (mapcar 'car changes)))
+      (funcall fun (point-min) (point-max))
+      (should (equal (buffer-string) (apply 'concat (mapcar 'cadr changes)))))))
+
+(ert-deftest tabify-tests-untabify ()
+  (let ((changes '(("***\n"        "***\n")
+                   (" ***\n"       " ***\n")
+                   ("\t***\n"      "  ***\n")
+                   ("\t ***\n"     "   ***\n")
+                   ("\t\t***\n"    "    ***\n")
+                   ("\t\t ***\n"   "     ***\n")
+                   ("\t\t\t***\n"  "      ***\n")
+                   ("\t\t\t ***\n" "       ***\n")
+                   ("  ***\n"      "  ***\n")
+                   (" \t ***\n"    "   ***\n")
+                   ("  \t***\n"    "    ***\n")
+                   ("   \t***\n"   "    ***\n"))))
+    (tabify-tests--test-changes 'untabify changes 2)))
+
+(ert-deftest tabify-tests-tabify ()
+  (let ((changes '(("***\n"        "***\n")
+                   (" ***\n"       " ***\n")
+                   ("  ***\n"      "\t***\n")
+                   ("   ***\n"     "\t ***\n")
+                   ("    ***\n"    "\t\t***\n")
+                   ("     ***\n"   "\t\t ***\n")
+                   ("      ***\n"  "\t\t\t***\n")
+                   ("       ***\n" "\t\t\t ***\n")
+                   ("\t***\n"      "\t***\n")
+                   ("\t ***\n"     "\t ***\n")
+                   ("\t\t***\n"    "\t\t***\n"))))
+    (tabify-tests--test-changes 'tabify changes 2)))
+
+(ert-deftest tabify-tests-tabify/all-spaces-on-line ()
+  (with-temp-buffer
+    (let ((tab-width 2))
+      (insert "  **  **  ")
+      (tabify (point-min) (point-max))
+      (should (equal (buffer-string) "\t**\t**\t")))))
+
+(ert-deftest tabify-tests-tabify/only-initial ()
+  (with-temp-buffer
+    (let ((tab-width 2)
+          (tabify-regexp "^\\t* [ \\t]+"))   ; from tabify-regexp docstring
+      (insert "  **  ")
+      (tabify (point-min) (point-max))
+      (should (equal (buffer-string) "\t**  ")))))
+
+(provide 'tabify-tests)
+;;; tabify-tests.el ends here
-- 
2.11.0

